//******************************************************************************
//
// OpenGL Utility / OGLH
//
// ヘルパ関数クラス
//
// Copyright (C) 2010-2012 WADA Masashi. All Rights Reserved.
//
//******************************************************************************

#import "OGLH.h"
#import <math.h>


//******************************************************************************
// パラメータ定義
//******************************************************************************
//円周率：floatの有効桁数は10進数で7桁ほど
#define OGL_MATH_PI  (3.141592)

//******************************************************************************
// ベクトル正規化
//******************************************************************************
void OGLH::Vec3Normalize(
		OGLVECTOR3* pNormalizedVector,
		const OGLVECTOR3* pVector
	)
{
	double x, y, z, r, ri;
	
	x = (double)(pVector->x);
	y = (double)(pVector->y);
	z = (double)(pVector->z);
	r = sqrt((x*x) + (y*y) + (z*z));
	
	if (r < (1e-6)) {
		//正規化できないので同じベクトルを返す
		ri = 1.0;
	}
	else {
		ri = 1.0 / r;
	}
	
	pNormalizedVector->x = (float)(x * ri);
	pNormalizedVector->y = (float)(y * ri);
	pNormalizedVector->z = (float)(z * ri);
}

//******************************************************************************
// ベクトル外積
//******************************************************************************
void OGLH::Vec3Cross(
		OGLVECTOR3* pOutVector,
		const OGLVECTOR3* pInVector1,
		const OGLVECTOR3* pInVector2
	)
{
	OGLVECTOR3 crossVector;
	
	crossVector.x = (pInVector1->y * pInVector2->z) - (pInVector1->z * pInVector2->y);
	crossVector.y = (pInVector1->z * pInVector2->x) - (pInVector1->x * pInVector2->z);
	crossVector.z = (pInVector1->x * pInVector2->y) - (pInVector1->y * pInVector2->x);
	
	*pOutVector = crossVector;
}

//******************************************************************************
// ラジアン算出
//******************************************************************************
float OGLH::ToRadian(float degree)
{
	return (degree * OGL_MATH_PI / 180.0);
}


